package com.sf.service.gateway.domain.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.sf.service.gateway.domain.*;
import com.sf.vertx.enums.GatewayServiceModel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.StringUtils.EMPTY;

/**
 * 服务详细信息
 *
 * @author zoukun
 * @date 2024-04-22
 */
@Data
@ApiModel(description = "服务详细信息展示对象")
public class GatewayServerDetailedVO {
    private static final long serialVersionUID = 1L;

    private Long id;

    @ApiModelProperty("服务名称")
    private String serverName;

    @ApiModelProperty("服务地址，路由模式为路由名称")
    private String serverAddress;

    @ApiModelProperty("备注")
    private String remark;

    @ApiModelProperty("服务状态")
    private String status;

    @ApiModelProperty("创建者")
    private String created;

    @ApiModelProperty("更新者")
    private String modified;

    @ApiModelProperty("服务模式（NORMAL=普通模式，ROUTE=路由模式）")
    private String serviceModel;

    @ApiModelProperty(" 服务类型，SAC=SAC规范服务，OPEN=开放服务")
    private String serviceType;

    @ApiModelProperty("路由规则id")
    private Long routeId;

    @ApiModelProperty("超时时间，单位毫秒,1-150000之间")
    private Integer timeout;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    @ApiModelProperty("包含的接口信息")
    private List<GatewayInterfaceInfoListVO> interfaceInfoList;

    public static GatewayServerDetailedVO convert(GatewayServer gatewayServer
            , GatewayRoute route
            , List<GatewayInterfaceInfo> interfaceInfoList
            , List<GatewayInterfaceLinkStrategy> linkList
            , List<GatewayStrategy> strategyList
    ) {
        if (gatewayServer == null) {
            return null;
        }
        Map<Long, List<GatewayInterfaceLinkStrategy>> groupByInterfaceLinkMap = linkList.stream().collect(Collectors.groupingBy(GatewayInterfaceLinkStrategy::getInterfaceId));
        Map<Long, GatewayStrategy> strategyMap = strategyList.stream().collect(Collectors.toMap(GatewayStrategy::getId, Function.identity()));
        GatewayServerDetailedVO vo = new GatewayServerDetailedVO();
        if (GatewayServiceModel.ROUTE.getCode().equals(gatewayServer.getServiceModel())) {
            vo.setServerAddress(route != null ? route.getRouteName() : EMPTY);
        } else {
            vo.setServerAddress(gatewayServer.getServerAddress());
        }
        vo.setId(gatewayServer.getId());
        vo.setServerName(gatewayServer.getServerName());
        vo.setRemark(gatewayServer.getRemark());
        vo.setStatus(gatewayServer.getStatus());
        vo.setCreated(gatewayServer.getCreated());
        vo.setModified(gatewayServer.getModified());
        vo.setServiceModel(gatewayServer.getServiceModel());
        vo.setServiceType(gatewayServer.getServiceType());
        vo.setRouteId(gatewayServer.getRouteId());
        vo.setTimeout(gatewayServer.getTimeout());
        vo.setCreateTime(gatewayServer.getCreateTime());
        vo.setUpdateTime(gatewayServer.getUpdateTime());
        List<GatewayInterfaceInfoListVO> interfaceInfoListVOList = interfaceInfoList.stream().map(info -> {
            GatewayInterfaceInfoListVO interfaceInfoListVO = GatewayInterfaceInfoListVO.convert(info,gatewayServer);
            // 策略信息
            List<GatewayInterfaceLinkStrategy> currentLinks = groupByInterfaceLinkMap.getOrDefault(info.getId(), Collections.emptyList());
            List<GatewayStrategyListVO> strategyListVOList = currentLinks.stream()
                    .map(link -> GatewayStrategyListVO.convert(strategyMap.get(link.getStrategyId())))
                    .collect(Collectors.toList());
            interfaceInfoListVO.setStrategyList(strategyListVOList);
            return interfaceInfoListVO;
        }).collect(Collectors.toList());
        vo.setInterfaceInfoList(interfaceInfoListVOList);
        return vo;
    }

}
